/**
 * @file scep.c
 * @brief SCEP specific functions
 * 
 * Contains functions to build SCEP request's and to parse SCEP reply's.
 */

/*
 * Copyright (C) 2005 Jan Hutter, Martin Willi
 * Hochschule fuer Technik Rapperswil
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <string.h>

#include <freeswan.h>

#include "../pluto/constants.h"
#include "../pluto/defs.h"
#include "../pluto/rnd.h"
#include "../pluto/asn1.h"
#include "../pluto/pkcs1.h"

#include "scep.h"

static char ASN1_messageType_oid_str[] = {
    0x06, 0x0A, 0x60, 0x86, 0x48, 0x01, 0x86, 0xF8, 0x45, 0x01, 0x09, 0x02
};

static char ASN1_senderNonce_oid_str[] = {
    0x06, 0x0A, 0x60, 0x86, 0x48, 0x01, 0x86, 0xF8, 0x45, 0x01, 0x09, 0x05
};

static char ASN1_transId_oid_str[] = {
    0x06, 0x0A, 0x60, 0x86, 0x48, 0x01, 0x86, 0xF8, 0x45, 0x01, 0x09, 0x07
};

static const chunk_t ASN1_messageType_oid =
			strchunk(ASN1_messageType_oid_str);
static const chunk_t ASN1_senderNonce_oid =
			strchunk(ASN1_senderNonce_oid_str);
static const chunk_t ASN1_transId_oid =
			strchunk(ASN1_transId_oid_str);

/*
 * builds a transId attribute
 */
chunk_t
scep_transId_attribute(chunk_t transaction_id)
{
    chunk_t transId;
    u_char buf[BUF_LEN];

    datatot(transaction_id.ptr, transaction_id.len, 16, buf, BUF_LEN);
    setchunk(transId, buf, strlen(buf));

    return asn1_wrap(ASN1_SEQUENCE, "cm"
		, ASN1_transId_oid
		, asn1_wrap(ASN1_SET, "m"
		    , asn1_simple_object(ASN1_PRINTABLESTRING, transId)
		  )
	      );
}

/*
 * builds a messageType attribute
 */
chunk_t
scep_messageType_attribute(void)
{
    chunk_t messageType = { "20", 2 };

    return asn1_wrap(ASN1_SEQUENCE, "cm"
		, ASN1_messageType_oid
		, asn1_wrap(ASN1_SET, "m"
		    , asn1_simple_object(ASN1_PRINTABLESTRING, messageType)
		  )
	      );
}

/*
 * builds a senderNonce attribute
 */
chunk_t
scep_senderNonce_attribute(void)
{
    const size_t nonce_len = 16;
    u_char nonce_buf[nonce_len];
    chunk_t senderNonce = { nonce_buf, nonce_len };

    get_rnd_bytes(nonce_buf, nonce_len);
 
    return asn1_wrap(ASN1_SEQUENCE, "cm"
		, ASN1_senderNonce_oid
		, asn1_wrap(ASN1_SET, "m"
		    , asn1_simple_object(ASN1_OCTET_STRING, senderNonce)
		  )
	      );
}
